﻿local addonName, addon = ...

do
	local exceptions = {
		RUNE_REGEN = {
			DEATHKNIGHT = true
		},
		FOCUS_REGEN = {
			HUNTER = true
		},
		DRUIDMANA = {
			DRUID = true,
		},
		ENERGY_REGEN = {
			ROGUE = true,
			DRUID = true,
		},
	}
	
	local _, class = UnitClass("player")
	for k, v in pairs(PAPERDOLL_STATCATEGORIES) do
		local stats = v.stats
		for i = #stats, 1, -1 do
			local v = exceptions[stats[i]]
			if v and not v[class] then
				tremove(stats, i)
			end
		end
	end
end

local DEFAULT_CATEGORY_ORDER = CopyTable(PAPERDOLL_STATCATEGORY_DEFAULTORDER)
local DEFAULT_CATEGORIES = CopyTable(PAPERDOLL_STATCATEGORIES)

local db
local statsDB = addon.statsDB
local cvarOrder, cvarCollapsed = "statCategoryOrder", "statCategoriesCollapsed"

addon = CreateFrame("Frame", "PDFStats", UIParent)
addon:SetToplevel(true)
addon:EnableMouse(true)
addon:SetMovable(true)
addon:SetSize(256, 56)
addon:SetPoint("CENTER")
addon:Hide()
addon:SetScript("OnMouseDown", addon.StartMoving)
addon:SetScript("OnMouseUp", addon.StopMovingOrSizing)
addon:SetBackdrop({
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = {left = 4, right = 4, top = 4, bottom = 4}
})
addon:SetBackdropColor(0, 0, 0, 0.8)
addon:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)
addon:RegisterEvent("ADDON_LOADED")
addon:RegisterEvent("PLAYER_LOGIN")
addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)

local debug = CreateFrame("Frame", nil, UIParent, "BasicFrameTemplate")
do
	debug:SetSize(384, 512)
	debug:SetPoint("RIGHT", -32, 0)
	
	debug.order = debug:CreateFontString(nil, nil, "ChatFontSmall")
	debug.order:SetPoint("TOPLEFT", 8, -32)
	debug.order:SetJustifyH("LEFT")
	
	debug.ids = debug:CreateFontString(nil, nil, "ChatFontSmall")
	debug.ids:SetPoint("TOPLEFT", debug, "LEFT", 8, 0)
	debug.ids:SetJustifyH("LEFT")
	
	debug.registry = debug:CreateFontString(nil, nil, "ChatFontSmall")
	debug.registry:SetPoint("TOPLEFT", debug, "TOP", 8, -32)
	debug.registry:SetJustifyH("LEFT")
	
	debug.collapse = debug:CreateFontString(nil, nil, "ChatFontSmall")
	debug.collapse:SetPoint("TOPLEFT", debug, "CENTER", 8, 0)
	debug.collapse:SetJustifyH("LEFT")
	
	function debug:Update()
		local order = "• DEFAULTORDER"
		local ids = "• IDs"
		local collapse = "• db.collapsed"
		for i, v in ipairs(PAPERDOLL_STATCATEGORY_DEFAULTORDER) do
			order = order..format("\n[%d] = %s", i, v)
			ids = ids..format("\n%s = %d", v, PAPERDOLL_STATCATEGORIES[v].id)
			collapse = collapse..format("\n[%d] = %s", i, tostring(db.collapsed[i]))
		end
		self.order:SetText(order.."\n\nCVar: "..GetCVar(cvarOrder))
		self.ids:SetText(ids)
		self.collapse:SetText(collapse)
		
		local registry = "• db.registry"
		for i, v in ipairs(db.registry) do
			registry = registry..format("\n[%d] = %s", i, v)
		end
		self.registry:SetText(registry)
	end
	
	debug:Hide()
end

local dualProfile = CreateFrame("CheckButton", nil, addon, "OptionsBaseCheckButtonTemplate")
dualProfile:SetPoint("BOTTOMLEFT")
dualProfile:SetPushedTextOffset(0, 0)
dualProfile.onClick = function(self)
	local checked = not not self:GetChecked()
	addon.db.dualProfile = checked
	local profile = addon.db.profile
	if checked then
		addon:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
		-- copy the primary spec profile to secondary spec profile if it doesn't exist
		profile[2] = profile[2] or CopyTable(profile[1])
		addon:CheckTalentSpec()
	else
		addon:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
		if profile[2] then
			profile[1] = profile[2]
			profile[2] = nil
		end
		db = profile[1]
	end
	debug:Update()
end
dualProfile:SetScript("OnClick", dualProfile.onClick)
dualProfile.tooltipText = "If disabled, will use the same layout for both primary and secondary talents."

local text = dualProfile:CreateFontString(nil, nil, "GameFontHighlight")
text:SetPoint("LEFT", dualProfile, "RIGHT", 0, 1)
dualProfile:SetFontString(text)
dualProfile:SetText("Use dual profile")

local frameList = CreateFrame("Frame", "PDFStatsFrameList", addon, "UIDropDownMenuTemplate")
frameList:SetPoint("TOPLEFT", -8, -8)
frameList.statLists = {}


local closeButton = CreateFrame("Button", nil, addon, "UIPanelCloseButton")
closeButton:SetPoint("TOPRIGHT", -2, -4)


local deleteButton = CreateFrame("Button", nil, addon, "UIPanelButtonTemplate")
deleteButton:SetSize(80, 22)
deleteButton:SetPoint("RIGHT", closeButton, "LEFT", 0, -1)
deleteButton:SetText(DELETE)
deleteButton:Hide()
deleteButton:SetScript("OnClick", function()
	addon:RemoveCategory(UIDropDownMenu_GetSelectedValue(frameList))
end)


local hideButton = CreateFrame("Button", nil, addon, "UIPanelButtonTemplate")
hideButton:SetSize(80, 22)
hideButton:SetPoint("RIGHT", closeButton, "LEFT", 0, -1)
hideButton:SetText(HIDE)
hideButton:Hide()


local resetButton = CreateFrame("Button", nil, addon, "UIPanelButtonTemplate")
resetButton:SetSize(80, 22)
resetButton:SetPoint("RIGHT", closeButton, "LEFT", 0, -1)
resetButton:SetText(RESET)
resetButton:Hide()
hideButton:SetScript("OnClick", function(self)
	local category = UIDropDownMenu_GetSelectedValue(frameList)
	addon:HideCategory(category)
	-- addon:RefreshCategory()
	addon:Print(format("Hid category '%s'.", _G["STAT_CATEGORY_"..category]))
	self:Hide()
	resetButton:Show()
end)
resetButton:SetScript("OnClick", function(self)
	addon:ResetCategory(UIDropDownMenu_GetSelectedValue(frameList))
	self:Hide()
	hideButton:Show()
end)


local editBox = CreateFrame("EditBox", "PDFStatsEditBox", addon, "InputBoxTemplate")
editBox:SetAutoFocus(true)
editBox:SetFontObject(ChatFontSmall)
editBox:SetSize(160, 20)
editBox:SetPoint("TOPLEFT", frameList, "BOTTOM", 4, 0)
editBox:SetText("Enter category name...")
editBox:Hide()
editBox:SetScript("OnEnterPressed", function(self)
	local text = self:GetText()
	if text:match("%S+") then
		if not addon:AddCategory(text) then
			self:SetText("Enter category name...")
			self:HighlightText()
			return
		end
		addon:SelectCategory(text)
		UIDropDownMenu_SetText(frameList, text)
		self:Hide()
	else
		self:SetText("Enter category name...")
		self:HighlightText()
		addon:Print("Invalid category name.")
	end
end)
editBox:SetScript("OnEscapePressed", function(self)
	self:Hide()
	addon:SelectCategory(nil)
end)

-- onClick function for all dropdown menu items
local function onClick(self)
	local id = self.owner.id
	addon:SetStat(id, self.value)
	local stats = addon:GetSelectedStats()
	if self.value ~= "none" then
		local n = #stats
		stats[id] = self.value
		if id > n then
			addon:UpdateArrows()
			addon:SetStat(n + 2)
			addon:SetHeight(56 + 28 * (n + 2))
		end
	else
		tremove(stats, id)
		addon:RefreshCategory()
	end
	local category = UIDropDownMenu_GetSelectedValue(frameList)
	if DEFAULT_CATEGORIES[category] then
		-- PAPERDOLL_STATCATEGORIES[category].stats = stats
		-- db.categories[category] = stats
		-- db.hidden[category] = true
		resetButton:Enable()
	end
	CloseDropDownMenus()
	PaperDollFrame_UpdateStats()
end

-- dropdown menu initialize function that adds all the menu items
local function initialize(self, level)
	if level == 1 then
		-- menu level 1; categories
		local info = UIDropDownMenu_CreateInfo()
		info.text = NONE
		info.value = "none"
		info.func = onClick
		info.owner = self
		UIDropDownMenu_AddButton(info, level)
		
		for i, v in ipairs(DEFAULT_CATEGORY_ORDER) do
			local info = UIDropDownMenu_CreateInfo()
			info.text = _G["STAT_CATEGORY_"..v]
			info.value = v
			info.notCheckable = true
			-- info.notClickable = true
			info.hasArrow = true
			UIDropDownMenu_AddButton(info, level)
		end
	elseif level == 2 then
		-- menu level 2; stats
		local info = UIDropDownMenu_CreateInfo()
		info.text = _G["STAT_CATEGORY_"..UIDROPDOWNMENU_MENU_VALUE]
		info.isTitle = true
		info.notCheckable = true
		UIDropDownMenu_AddButton(info, level)
		
		local category = statsDB[UIDROPDOWNMENU_MENU_VALUE]
		for i, stat in ipairs(category) do
			local info = UIDropDownMenu_CreateInfo()
			info.text = stat.label
			info.value = stat.value
			info.func = onClick
			info.owner = self
			UIDropDownMenu_AddButton(info, level)
		end
	end
end

local function onEnter(self) self:SetAlpha(0.6) end
local function onLeave(self) self:SetAlpha(0.4) end

local function createArrow(parent)
	local btn = CreateFrame("Button", nil, parent)
	btn:SetSize(16, 8)
	btn:SetAlpha(0.4)
	btn:SetScript("OnEnter", onEnter)
	btn:SetScript("OnLeave", onLeave)
	local tx = btn:CreateTexture(nil, "BACKGROUND")
	tx:SetTexture([[Interface\PaperDollInfoFrame\StatSortArrows]])
	tx:SetAllPoints()
	btn.tx = tx
	btn:SetNormalTexture(tx)
	return btn
end

local function onUpClick(self)
	local stats = addon:GetSelectedStats()
	local id = self:GetParent().id
	stats[id], stats[id - 1] = stats[id - 1], stats[id]
	addon:RefreshCategory()
	PaperDollFrame_UpdateStats()
end

local function onDownClick(self)
	local stats = addon:GetSelectedStats()
	local id = self:GetParent().id
	stats[id], stats[id + 1] = stats[id + 1], stats[id]
	addon:RefreshCategory()
	PaperDollFrame_UpdateStats()
end

function addon:NewStat(index)
	local frame = CreateFrame("Frame", "PDFStatsFrameListStat"..index, self, "UIDropDownMenuTemplate")
	if index == 1 then
		frame:SetPoint("TOPLEFT", frameList, "BOTTOMLEFT", 0, -8)
	else
		frame:SetPoint("TOP", frameList.statLists[index - 1], "BOTTOM", 0, 4)
	end
	frame.id = index
	frame:Hide()
	
	UIDropDownMenu_Initialize(frame, initialize)
	
	UIDropDownMenu_SetWidth(frame, 192)
	UIDropDownMenu_JustifyText(frame, "LEFT")
	UIDropDownMenu_SetSelectedValue(frame, "none")

	local up = createArrow(frame)
	up:SetPoint("BOTTOMLEFT", frame, "RIGHT", -8, 2)
	up.tx:SetTexCoord(0, 1, 0, 0.5)
	up:SetScript("OnClick", onUpClick)
	frame.up = up
	if index == 1 then up:Hide() end

	local down = createArrow(frame)
	down:SetPoint("TOP", up, "BOTTOM")
	down.tx:SetTexCoord(0, 1, 0.5, 1)
	down:SetScript("OnClick", onDownClick)
	frame.down = down

	frameList.statLists[index] = frame
	return frame
end


local function newCategory()
	addon:SelectCategory(nil)
	editBox:SetText("Enter category name...")
	editBox:Show()
end

local function onClick(self)
	addon:SelectCategory(self.value)
	editBox:Hide()
end

function frameList:initialize()
	-- add default stat frames
	for i, v in ipairs(DEFAULT_CATEGORY_ORDER) do
		local info = UIDropDownMenu_CreateInfo()
		info.text = _G["STAT_CATEGORY_"..v]
		info.value = v
		info.func = onClick
		info.colorCode = not PAPERDOLL_STATCATEGORIES[v] and GRAY_FONT_COLOR_CODE
		UIDropDownMenu_AddButton(info)
	end

	for i, v in ipairs(db.registry) do
		if not DEFAULT_CATEGORIES[v] then
			local info = UIDropDownMenu_CreateInfo()
			info.text = v
			info.func = onClick
			info.colorCode = "|cff56a3ff"
			UIDropDownMenu_AddButton(info)
		end
	end
	
	local info = UIDropDownMenu_CreateInfo()
	info.text = "Create new..."
	info.value = "new"
	info.func = newCategory
	info.notCheckable = true
	UIDropDownMenu_AddButton(info)
end

UIDropDownMenu_JustifyText(frameList, "LEFT")
UIDropDownMenu_SetText(frameList, "Select category")

addon:NewStat(1)
addon:SetHeight(56 + 28)


--[[
local orig_GetCVar = GetCVar

function GetCVar(cvar)
	if cvar == cvarOrder then
		return db.statCategoryOrder
	end
	return orig_GetCVar(cvar)
end

local orig_SetCVar = SetCVar
local orig_GetCVarBitfield = GetCVarBitfield

function GetCVarBitfield(cvar, bit)
	if cvar == cvarCollapsed then
		return db.collapsed[bit]
	end
	return orig_GetCVarBitfield(cvar, bit)
end
]]

hooksecurefunc("SetCVar", function(cvar, value)
	if cvar == cvarOrder then
		db.statCategoryOrder = value
		debug:Update()
	end
end)

hooksecurefunc("SetCVarBitfield", function(cvar, bit, value)
	if cvar == cvarCollapsed then
		db.collapsed[bit] = bit > #PAPERDOLL_STATCATEGORY_DEFAULTORDER and nil or value
		debug:Update()
	end
end)


SlashCmdList.PDFSTATS_TOGGLE = function()
	if addon:IsShown() then
		addon:Hide()
	else
		addon:Show()
	end
end
SLASH_PDFSTATS_TOGGLE1 = "/pdf"
SLASH_PDFSTATS_TOGGLE2 = "/stats"


local defaults = {
	profile = {
		{
			registry = CopyTable(DEFAULT_CATEGORY_ORDER),
			categories = {},
			statCategoryOrder = "",
			collapsed = {},
		}
	},
	dualProfile = true,
}

for i, v in ipairs(DEFAULT_CATEGORY_ORDER) do
	defaults.profile[1].categories[v] = DEFAULT_CATEGORIES[v].stats
end

function addon:ADDON_LOADED(addon)
	if addon == addonName then
		PDFStatsDB = PDFStatsDB or defaults
		self.db = PDFStatsDB
		db = PDFStatsDB
		
		if not db.profile then
			PDFStatsDB = {dualProfile = true, profile = {db}}
		end
		
		self.db = PDFStatsDB
		db = PDFStatsDB
		
		self:UnregisterEvent("ADDON_LOADED")
	end
end


function addon:PLAYER_LOGIN()
	-- inserting this on PLAYER_LOGIN since UnitPowerType doesn't return anything earlier
	for i, v in ipairs(statsDB["GENERAL"]) do
		if v.value == "POWER" then
			v.label = _G[select(2, UnitPowerType("player"))]
			break
		end
	end
	
	dualProfile:SetChecked(self.db.dualProfile)
	dualProfile:onClick()
		
	self:LoadSettings()
end


function addon:LoadSettings()
	for i, name in ipairs(DEFAULT_CATEGORY_ORDER) do
		if not db.categories[name] then
			local id = PAPERDOLL_STATCATEGORIES[name].id
			
			for i = 1, #PAPERDOLL_STATCATEGORY_DEFAULTORDER do
				local v = PAPERDOLL_STATCATEGORY_DEFAULTORDER[i]
				if v == name then
					tremove(PAPERDOLL_STATCATEGORY_DEFAULTORDER, i)
				end
				
				local v2 = PAPERDOLL_STATCATEGORIES[PAPERDOLL_STATCATEGORY_DEFAULTORDER[i]]
				if v2 and v2.id > id then
					v2.id = v2.id - 1
				end
				if db.registry[i] == name then
					tremove(db.registry, i)
				end
			end
			PAPERDOLL_STATCATEGORIES[name] = nil
		end
	end
	
	-- insert our stored custom categories into table of default categories
	for i, v in ipairs(db.registry) do
		if DEFAULT_CATEGORIES[v] then
			PAPERDOLL_STATCATEGORIES[v].id = i
			PAPERDOLL_STATCATEGORIES[v].stats = db.categories[v]
		else
			self:AddCategory(v)
		end
	end
	
	if db.statCategoryOrder == "" then
		local order = {}
		for i = 1, #PAPERDOLL_STATCATEGORY_DEFAULTORDER do
			order[i] = i
		end
		db.statCategoryOrder = table.concat(order, ",")
	end
	
	SetCVar(cvarOrder, db.statCategoryOrder)
	
	for i, v in ipairs(PAPERDOLL_STATCATEGORY_DEFAULTORDER) do
		SetCVarBitfield(cvarCollapsed, i, db.collapsed[i] or false)
	end
	
	PaperDoll_InitStatCategories(PAPERDOLL_STATCATEGORY_DEFAULTORDER, cvarOrder, cvarCollapsed, "player")
	debug:Update()
end


function addon:CheckTalentSpec(activeSpec)
	activeSpec = activeSpec or GetActiveTalentGroup()
	if activeSpec == 1 then
		cvarOrder = "statCategoryOrder"
		cvarCollapsed = "statCategoriesCollapsed"
	else
		cvarOrder = "statCategoryOrder_2"
		cvarCollapsed = "statCategoriesCollapsed_2"
	end
	if self.db.dualProfile then
		db = self.db.profile[activeSpec]
	end
end


function addon:ACTIVE_TALENT_GROUP_CHANGED(activeSpec)
	self:CheckTalentSpec(activeSpec)
	PAPERDOLL_STATCATEGORY_DEFAULTORDER = CopyTable(DEFAULT_CATEGORY_ORDER)
	PAPERDOLL_STATCATEGORIES = CopyTable(DEFAULT_CATEGORIES)
	self:LoadSettings()
end


-- given the value, retrieves data about a stat from the stats table
function addon:GetStatInfo(value)
	for category, stats in pairs(statsDB) do
		for i, stat in ipairs(stats) do
			if stat.value == value then
				return stat.label, _G["STAT_CATEGORY_"..category]
			end
		end
	end
end


-- adds a new custom category
function addon:AddCategory(name)
	if PAPERDOLL_STATCATEGORIES[name] or DEFAULT_CATEGORIES[name] then
		self:Print("A frame with this name already exists. Please choose another name.")
		return
	end
	local id = #PAPERDOLL_STATCATEGORY_DEFAULTORDER + 1
	if not _G["CharacterStatsPaneCategory"..id] then
		CreateFrame("Frame", "CharacterStatsPaneCategory"..id, CharacterStatsPaneScrollChild, "StatGroupTemplate")
	end
	local categories = db.categories
	if not categories[name] then
		categories[name] = {}
		db.registry[id] = name
	end
	PAPERDOLL_STATCATEGORIES[name] = {id = id, stats = categories[name]}
	PAPERDOLL_STATCATEGORY_DEFAULTORDER[id] = name
	_G["STAT_CATEGORY_"..name] = name
	self:CVarAppendID(id)
	PaperDoll_InitStatCategories(PAPERDOLL_STATCATEGORY_DEFAULTORDER, cvarOrder, cvarCollapsed, "player")
	debug:Update()
	return true
end


-- resets default stat categories, including visibility and stats
function addon:ResetCategory(name)
	local category = DEFAULT_CATEGORIES[name]
	local categoryCopy = CopyTable(category)
	PAPERDOLL_STATCATEGORIES[name] = categoryCopy
	if self:IsHiddenCategory(name) then
	db.categories[name] = categoryCopy.stats
		local i = #PAPERDOLL_STATCATEGORY_DEFAULTORDER + 1
		-- need to create a new frame if we didn't have this many frames before
		if not _G["CharacterStatsPaneCategory"..i] then
			CreateFrame("Frame", "CharacterStatsPaneCategory"..i, CharacterStatsPaneScrollChild, "StatGroupTemplate")
		end
		categoryCopy.id = i
		tinsert(PAPERDOLL_STATCATEGORY_DEFAULTORDER, i, name)
		db.registry[i] = name
		self:CVarAppendID(i)
		PaperDoll_InitStatCategories(PAPERDOLL_STATCATEGORY_DEFAULTORDER, cvarOrder, cvarCollapsed, "player")
	else
	db.categories[name] = categoryCopy.stats
		PaperDollFrame_UpdateStats()
	end
	debug:Update()
end


-- removes a custom category from the database
function addon:RemoveCategory(name)
	_G["STAT_CATEGORY_"..name] = nil
	self:SelectCategory(nil)
	self:CVarRemoveID(name)
	self:Print(format("Removed category '%s'.", name))
	PaperDoll_InitStatCategories(PAPERDOLL_STATCATEGORY_DEFAULTORDER, cvarOrder, cvarCollapsed, "player")
	debug:Update()
end


-- hides default stat categories
function addon:HideCategory(name)
	self:CVarRemoveID(name)
	PaperDoll_InitStatCategories(PAPERDOLL_STATCATEGORY_DEFAULTORDER, cvarOrder, cvarCollapsed, "player")
	debug:Update()
end


local categoryOrder = {}

function addon:GetCategoryOrder()
	wipe(categoryOrder)
	local i = 0
	for id in gmatch(GetCVar(cvarOrder), "%d+") do
		i = i + 1
		categoryOrder[i] = tonumber(id)
	end
	return categoryOrder
end

local collapsedCategories = {}

function addon:GetCollapsedCategories()
	wipe(collapsedCategories)
	for i in ipairs(PAPERDOLL_STATCATEGORY_DEFAULTORDER) do
		collapsedCategories[i] = db.collapsed[i]--GetCVarBitfield(cvarCollapsed, i)
	end
	return collapsedCategories
end

function addon:CVarAppendID(id)
	local categoryOrder = GetCVar(cvarOrder)
	if #categoryOrder > 0 and not (","..categoryOrder..","):match(","..id..",") then
		SetCVar(cvarOrder, format("%s,%s", GetCVar(cvarOrder), id))
	end
	SetCVarBitfield(cvarCollapsed, id, false)
end


function addon:CVarRemoveID(name)
	local id = PAPERDOLL_STATCATEGORIES[name].id
	
	local categoryOrder = self:GetCategoryOrder()
	if #categoryOrder ~= 0 then
		for i = 1, #categoryOrder do
			-- remove the index whose value equals the ID of the removed frame
			local v = categoryOrder[i]
			if v == id then
				tremove(categoryOrder, i)
			end
			-- and lower all values greater than the desired ID by 1
			local v = categoryOrder[i]
			if v and v > id then
				categoryOrder[i] = v - 1
			end
		end
		SetCVar(cvarOrder, table.concat(categoryOrder, ","))
	end
	local collapsed = self:GetCollapsedCategories()
	tremove(collapsed, id)
	tremove(db.collapsed, id)
	for i = id, #collapsed + 1 do
		SetCVarBitfield(cvarCollapsed, i, collapsed[i] or false)
	end
	
	db.categories[name] = nil
	for i = 1, #PAPERDOLL_STATCATEGORY_DEFAULTORDER do
		local v = PAPERDOLL_STATCATEGORY_DEFAULTORDER[i]
		if v == name then
			tremove(PAPERDOLL_STATCATEGORY_DEFAULTORDER, i)
		end
		
		local v2 = PAPERDOLL_STATCATEGORIES[PAPERDOLL_STATCATEGORY_DEFAULTORDER[i]]
		if v2 and v2.id > id then
			v2.id = v2.id - 1
		end
		if db.registry[i] == name then
			tremove(db.registry, i)
		end
	end
	PAPERDOLL_STATCATEGORIES[name] = nil
end


-- selects a category, setting appropriate values on all dropdowns
function addon:SelectCategory(name)
	deleteButton:Hide()
	hideButton:Hide()
	resetButton:Hide()
	if name then
		UIDropDownMenu_SetSelectedValue(frameList, name)
		if DEFAULT_CATEGORIES[name] then
			if not PAPERDOLL_STATCATEGORIES[name] or self:IsModifiedCategory(name) then
				if self:IsModifiedCategory(name) then
					self:RefreshCategory()
				end
				resetButton:Show()
			else
				self:RefreshCategory()
				hideButton:Show()
			end
		else
			self:RefreshCategory()
			deleteButton:Show()
		end
	else
		UIDropDownMenu_SetSelectedValue(frameList, nil)
		UIDropDownMenu_SetText(frameList, "Select category")
		for i, v in ipairs(frameList.statLists) do
			v:Hide()
		end
		self:SetHeight(56 + 28)
	end
end


function addon:RefreshCategory()
	local stats = self:GetSelectedStats()
	for i, v in ipairs(stats) do
		self:SetStat(i, v)
	end
	local n = #stats + 1
	-- add a frame at the bottom so we can add more stats
	self:SetStat(n)
	self:HideUnused()
	self:UpdateArrows()
	self:SetHeight(56 + 28 * (#stats + 1))
end


function addon:HideUnused()
	for i = #self:GetSelectedStats() + 2, #frameList.statLists do
		self:GetStatFrame(i):Hide()
	end
end


-- set a stat to the given frame
function addon:SetStat(index, statValue)
	local frame = self:GetStatFrame(index)
	local stat, category = addon:GetStatInfo(statValue)
	if stat then
		UIDropDownMenu_SetSelectedValue(frame, statValue)
		UIDropDownMenu_SetText(frame, format("%s - %s", category, stat))
	else
		UIDropDownMenu_SetSelectedValue(frame, "none")
		UIDropDownMenu_SetText(frame, NONE)
	end
	frame:Show()
end


-- gets an array of stat values based on the currently selected category
function addon:GetSelectedStats()
	return PAPERDOLL_STATCATEGORIES[UIDropDownMenu_GetSelectedValue(frameList)].stats
end


-- gets the dropdown frame for the given stat index, creates a new frame if necessary
function addon:GetStatFrame(index)
	return frameList.statLists[index] or self:NewStat(index)
end


function addon:IsHiddenCategory(name)
	if DEFAULT_CATEGORIES[name] and not db.categories[name] then
		return true
	end
	return false
end


function addon:IsModifiedCategory(name)
	if not (DEFAULT_CATEGORIES[name] and db.categories[name]) then
		return false
	end
	local default, copy = DEFAULT_CATEGORIES[name].stats, db.categories[name]
	if not #default == #copy then
		return true
	end
	for i = 1, max(#default, #copy) do
		if default[i] ~= copy[i] then
			return true
		end
	end
	return false
end


-- sets swap-position-arrows visibility
function addon:UpdateArrows()
	local n = #self:GetSelectedStats()
	if n > 0 then
		for i = 1, n do
			local frame = self:GetStatFrame(i)
			frame.up:Show()
			frame.down:Show()
		end
		self:GetStatFrame(1).up:Hide()
		self:GetStatFrame(n).down:Hide()
	end
	local frame = self:GetStatFrame(n + 1)
	frame.up:Hide()
	frame.down:Hide()
end


function addon:Print(msg)
	print("|cff56a3ffPDFStats:|r", msg)
end