﻿local _, addon = ...

local AVOIDANCE = "Avoidance"
local AVOIDANCE_BLOCK = "Avoidance w/ block"

local BLOCK_VALUE_TOOLTIP = CR_BLOCK_TOOLTIP:match("\n(.+)")

---------------------------------------
-- these are functions that were made
-- for "custom made" stat displays
---------------------------------------

local itemLevelSlots = {
	"BackSlot",
	"ChestSlot",
	"FeetSlot",
	"Finger0Slot",
	"Finger1Slot",
	"HandsSlot",
	"HeadSlot",
	"LegsSlot",
	"MainHandSlot",
	"NeckSlot",
	"RangedSlot",
	"ShoulderSlot",
	"Trinket0Slot",
	"Trinket1Slot",
	"WaistSlot",
	"WristSlot",
}

local offHandSlot = #itemLevelSlots + 1

PAPERDOLL_STATINFO["ITEMLEVEL_WORN"] = {
	updateFunc = function(statFrame, unit)
		local avgItemLevel = 0
		if IsEquippedItemType(INVTYPE_WEAPON) or IsEquippedItemType(INVTYPE_WEAPONMAINHAND) then
			itemLevelSlots[offHandSlot] = "SecondaryHandSlot"
		else
			itemLevelSlots[offHandSlot] = nil
		end
		for i, v in ipairs(itemLevelSlots) do
			local link = GetInventoryItemLink("player", GetInventorySlotInfo(v))
			if link then
				avgItemLevel = avgItemLevel + (select(4, GetItemInfo(link)) or 0)
			end
		end
		PaperDollFrame_SetLabelAndText(statFrame, STAT_AVERAGE_ITEM_LEVEL.." (equipped)", floor(avgItemLevel / #itemLevelSlots))
		statFrame:Show()
	end
}


PAPERDOLL_STATINFO["BLOCKVALUE"] = {
	updateFunc = function(statFrame, unit)
		local blockValue = GetShieldBlock()
		PaperDollFrame_SetLabelAndText(statFrame, ITEM_MOD_BLOCK_VALUE_SHORT, blockValue, 1)
		statFrame.tooltip = HIGHLIGHT_FONT_COLOR_CODE..format("%s %d%%", ITEM_MOD_BLOCK_VALUE_SHORT, blockValue)..FONT_COLOR_CODE_CLOSE
		statFrame.tooltip2 = BLOCK_VALUE_TOOLTIP:format(blockValue)
		statFrame:Show()
	end
}


-- this is your combined dodge, parry chance and chance to be missed
local function avoidance_OnEnter(self)
	local dodge, parry, miss = GetDodgeChance(), GetParryChance()
	local avoidance = dodge + parry-- + miss
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText(HIGHLIGHT_FONT_COLOR_CODE..format("Avoidance %.2f%%", avoidance)..FONT_COLOR_CODE_CLOSE)
	GameTooltip:AddDoubleLine(DODGE_CHANCE, format("%.2f%%", dodge))
	GameTooltip:AddDoubleLine(PARRY_CHANCE, format("%.2f%%", parry))
	
	GameTooltip:Show()
end

PAPERDOLL_STATINFO["AVOIDANCE"] = {
	updateFunc = function(statFrame, unit)
		local avoidance = GetDodgeChance() + GetParryChance()
		PaperDollFrame_SetLabelAndText(statFrame, AVOIDANCE, avoidance, 1)
		statFrame:SetScript("OnEnter", avoidance_OnEnter)
		statFrame:Show()
	end
}


local function avoidanceBlock_OnEnter(self)
	local dodge, parry, block, miss = GetDodgeChance(), GetParryChance(), GetBlockChance()
	local avoidance = dodge + parry + block
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText(HIGHLIGHT_FONT_COLOR_CODE..format("Avoidance %.2f%%", avoidance)..FONT_COLOR_CODE_CLOSE)
	GameTooltip:AddDoubleLine(DODGE_CHANCE, format("%.2f%%", dodge))
	GameTooltip:AddDoubleLine(PARRY_CHANCE, format("%.2f%%", parry))
	GameTooltip:AddDoubleLine(BLOCK_CHANCE, format("%.2f%%", block))
	GameTooltip:Show()
end

-- same as above but including block chance
PAPERDOLL_STATINFO["AVOIDANCE_BLOCK"] = {
	updateFunc = function(statFrame)
		local avoidance = GetDodgeChance() + GetBlockChance() + GetParryChance()
		PaperDollFrame_SetLabelAndText(statFrame, AVOIDANCE, avoidance, 1)
		statFrame:SetScript("OnEnter", avoidanceBlock_OnEnter)
		statFrame:Show()
	end
}


-- table of available stats
addon.statsDB = {
	GENERAL = {
		{value = "HEALTH",			label = HEALTH},
		{value = "DRUIDMANA",		label = MANA},
		{value = "POWER"},
		{value = "MASTERY",			label = STAT_MASTERY},
		{value = "ITEMLEVEL",		label = STAT_AVERAGE_ITEM_LEVEL},
		{value = "ITEMLEVEL_WORN",	label = STAT_AVERAGE_ITEM_LEVEL.." (equipped)"},
		{value = "MOVESPEED",		label = STAT_MOVEMENT_SPEED},
	},

	ATTRIBUTES = {
		{value = "STRENGTH",	label = SPELL_STAT1_NAME},
		{value = "AGILITY",		label = SPELL_STAT2_NAME},
		{value = "STAMINA",		label = SPELL_STAT3_NAME},
		{value = "INTELLECT",	label = SPELL_STAT4_NAME},
		{value = "SPIRIT",		label = SPELL_STAT5_NAME},
	},
	
	MELEE = {
		{value = "MELEE_DAMAGE",		label = DAMAGE},
		{value = "MELEE_DPS",			label = STAT_DPS_SHORT},
		{value = "MELEE_AP",			label = STAT_ATTACK_POWER},
		{value = "MELEE_ATTACKSPEED",	label = WEAPON_SPEED},
		{value = "HASTE",				label = STAT_HASTE},
		{value = "ENERGY_REGEN",		label = STAT_ENERGY_REGEN},
		{value = "RUNE_REGEN",			label = STAT_RUNE_REGEN},
		{value = "HITCHANCE",			label = STAT_HIT_CHANCE},
		{value = "CRITCHANCE",			label = MELEE_CRIT_CHANCE},
		{value = "EXPERTISE",			label = STAT_EXPERTISE},
		{value = "MISS_CHANCE0",		label = MISS_CHANCE},
	},
	
	RANGED = {
		{value = "RANGED_DAMAGE",		label = DAMAGE},
		{value = "RANGED_DPS",			label = STAT_DPS_SHORT},
		{value = "RANGED_AP",			label = STAT_ATTACK_POWER},
		{value = "RANGED_ATTACKSPEED",	label = WEAPON_SPEED},
		{value = "RANGED_HASTE",		label = STAT_HASTE},
		{value = "FOCUS_REGEN",			label = STAT_FOCUS_REGEN},
		{value = "RANGED_HITCHANCE",	label = STAT_HIT_CHANCE},
		{value = "RANGED_CRITCHANCE",	label = RANGED_CRIT_CHANCE},
	},
	
	SPELL = {
		{value = "SPELLDAMAGE",			label = STAT_SPELLDAMAGE},
		{value = "SPELLHEALING",		label = STAT_SPELLHEALING},
		{value = "SPELL_HASTE",			label = STAT_HASTE},
		{value = "SPELL_HITCHANCE",		label = STAT_HIT_CHANCE},
		{value = "SPELL_PENETRATION",	label = SPELL_PENETRATION},
		{value = "MANAREGEN",			label = MANA_REGEN},
		{value = "COMBATMANAREGEN",		label = MANA_REGEN_COMBAT},
		{value = "SPELLCRIT",			label = SPELL_CRIT_CHANCE},
	},
	
	DEFENSE = {
		{value = "ARMOR",					label = ARMOR},
		{value = "DODGE",					label = STAT_DODGE},
		{value = "PARRY",					label = STAT_PARRY},
		{value = "BLOCK",					label = STAT_BLOCK},
		{value = "BLOCKVALUE",				label = ITEM_MOD_BLOCK_VALUE_SHORT},
		{value = "RESILIENCE_REDUCTION",	label = STAT_RESILIENCE},	-- resilience
		-- {value = "RESILIENCE_CRIT",			label = STAT_RESILIENCE},	-- resilience
		{value = "AVOIDANCE",				label = AVOIDANCE},			-- avoidance (dodge + parry)
		{value = "AVOIDANCE_BLOCK",			label = AVOIDANCE_BLOCK},	-- avoidance w/ block
	},
	
	RESISTANCE = {},
}

for i, v in ipairs{6,2,3,4,5} do
	addon.statsDB["RESISTANCE"][i] = {value = PAPERDOLL_STATCATEGORIES["RESISTANCE"].stats[i], label = _G["SPELL_SCHOOL"..v.."_CAP"]}
end

local missRates = {
	{
		value = "MISS_CHANCE_SPECIAL",
		label = MISS_CHANCE.." special",
		func = GetMeleeMissChance,
		onEnter = MeleeHitChance_OnEnter,
		category = "MELEE",
		isSpecial = true,
	},
	{
		value = "MISS_CHANCE",
		label = MISS_CHANCE,
		func = GetMeleeMissChance,
		onEnter = MeleeHitChance_OnEnter,
		category = "MELEE",
	},
	{
		value = "RANGED_MISS_CHANCE",
		label = MISS_CHANCE,
		func = GetRangedMissChance,
		onEnter = RangedHitChance_OnEnter,
		category = "RANGED",
	},
	{
		value = "SPELL_MISS_CHANCE",
		label = MISS_CHANCE,
		func = GetSpellMissChance,
		onEnter = SpellHitChance_OnEnter,
		category = "SPELL",
	},
}

for i, v in ipairs(missRates) do
	for i = 0, 3 do
		local value = v.value..i
		local label = v.label.." +"..i
		PAPERDOLL_STATINFO[value] = {
			updateFunc = function(statFrame)
				PaperDollFrame_SetLabelAndText(statFrame, label, v.func(i, isSpecial), 1)
				statFrame:SetScript("OnEnter", v.onEnter)
				statFrame:Show()
			end
		}
		tinsert(addon.statsDB[v.category], {value = value, label = label})
	end
end

local enemyAvoidRates = {
	{value = "ENEMY_DODGE_CHANCE", label = STAT_DODGE, func = GetEnemyDodgeChance},
	{value = "ENEMY_PARRY_CHANCE", label = STAT_PARRY, func = GetEnemyParryChance},
}

for i, v in ipairs(enemyAvoidRates) do
	for i = 0, 3 do
		local value = v.value..i
		local label = ENEMY.." "..v.label.." +"..i
		PAPERDOLL_STATINFO[value] = {
			updateFunc = function(statFrame)
				PaperDollFrame_SetLabelAndText(statFrame, label, v.func(i), 1)
				statFrame:SetScript("OnEnter", Expertise_OnEnter)
				statFrame:Show()
			end
		}
		tinsert(addon.statsDB["MELEE"], {value = value, label = label})
	end
end